@@ -1,4 +1,7 @@
+from django.conf import settings
+from django.conf.urls.static import static
 from django.contrib import admin
+from django.contrib.staticfiles.urls import staticfiles_urlpatterns
 from django.urls import include, path
 from drf_yasg import openapi
 from drf_yasg.views import get_schema_view
@@ -23,4 +26,6 @@ urlpatterns = [
     path("admin/", admin.site.urls),
     path("ecom/", include("ob_dj_store.apis.stores.urls")),
     path("tap/", include("ob_dj_store.apis.tap.urls")),
-]
+] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
+
+urlpatterns += staticfiles_urlpatterns()
