@@ -1,10 +1,12 @@
-import contextlib
+import calendar
+import sys
 import typing
-from datetime import date, time, timedelta
+from datetime import time, timedelta
 
 from django.conf import settings
+from django.contrib.contenttypes.models import ContentType
 from django.contrib.gis.geos import Point
-from django.core.exceptions import ObjectDoesNotExist
+from django.core.exceptions import ObjectDoesNotExist, ValidationError
 from django.shortcuts import get_object_or_404
 from django.utils.timezone import now
 from django.utils.translation import gettext_lazy as _
@@ -49,6 +51,7 @@ class AttributeChoiceSerializer(serializers.ModelSerializer):
             "id",
             "name",
             "price",
+            "is_default",
         )
 
 
@@ -67,19 +70,22 @@ class AttributeSerializer(serializers.ModelSerializer):
 class InventoryValidationMixin:
     def validate(self, attrs: typing.Dict) -> typing.Dict:
         validated_data = super().validate(attrs)
-        if validated_data["product_variant"].has_inventory:
-            if validated_data["quantity"] < 1:
-                raise serializers.ValidationError(_("Quantity must be greater than 0."))
-            # validate quantity in inventory
+        inventory = None
+        try:
             inventory = validated_data["product_variant"].inventories.get(
                 store=validated_data["store"]
             )
-            if not inventory.is_uncountable:
-                stock_quantity = inventory.quantity
-                if validated_data["quantity"] > stock_quantity:
-                    raise serializers.ValidationError(
-                        _("Quantity is greater than the stock quantity.")
-                    )
+        except:
+            raise serializers.ValidationError(_("Product has no inventory"))
+        if validated_data["quantity"] < 1:
+            raise serializers.ValidationError(_("Quantity must be greater than 0."))
+        # validate quantity in inventory
+        if not inventory.is_uncountable:
+            stock_quantity = inventory.quantity
+            if validated_data["quantity"] > stock_quantity:
+                raise serializers.ValidationError(
+                    _("Quantity is greater than the stock quantity.")
+                )
         return validated_data
 
 
@@ -158,6 +164,7 @@ class OrderSerializer(serializers.ModelSerializer):
             "preparation_time",
             "estimated_timeline",
             "history",
+            "car_id",
             "pickup_time",
             "extra_infos",
             "created_at",
@@ -208,6 +215,9 @@ class OrderSerializer(serializers.ModelSerializer):
     def validate(self, attrs):
         user = self.context["request"].user
         attrs["store"] = self._get_store()
+        stores = Store.objects.filter(store_items__cart=user.cart).distinct()
+        if not store_settings.DIFFERENT_STORE_ORDERING and len(stores) > 1:
+            raise ValidationError(_("You cannot order from different stores"))
         if "extra_infos" in attrs:
             errors = []
             data = attrs["extra_infos"]
@@ -232,41 +242,48 @@ class OrderSerializer(serializers.ModelSerializer):
             raise serializers.ValidationError(_("The Cart must not be empty"))
 
         if "pickup_time" in attrs:
-            # validate that the pickup_time is always in the future
-            if attrs["pickup_time"] < now():
-                raise serializers.ValidationError(
-                    _("Pickup time must be in the future")
-                )
-            # validate that the pickup_time is part of day (between 00:00 and 23:59)
-            if not (
-                attrs["pickup_time"].time() >= time(hour=0, minute=0)
-                and attrs["pickup_time"].time() <= time(hour=23, minute=59)
-            ):
-                raise serializers.ValidationError(_("Pickup time must be part of day"))
-            # validate that the pickup_time is between store's opening hours and closing hours
-            if (
-                attrs["store"]
-                .opening_hours.filter(weekday=attrs["pickup_time"].weekday() + 1)
-                .exists()
-            ) and (
-                attrs["pickup_time"].hour
-                > (
+            if attrs["pickup_time"]:
+                # validate that the pickup_time is always in the future
+                if attrs["pickup_time"] < now():
+                    raise serializers.ValidationError(
+                        _("Pickup time must be in the future")
+                    )
+                # validate that the pickup_time is part of day (between 00:00 and 23:59)
+                if not (
+                    attrs["pickup_time"].time() >= time(hour=0, minute=0)
+                    and attrs["pickup_time"].time() <= time(hour=23, minute=59)
+                ):
+                    raise serializers.ValidationError(
+                        _("Pickup time must be part of day")
+                    )
+                # validate that the pickup_time is between store's opening hours and closing hours
+                if (
                     attrs["store"]
                     .opening_hours.filter(weekday=attrs["pickup_time"].weekday() + 1)
-                    .first()
-                    .to_hour.hour
-                )
-                or attrs["pickup_time"].hour
-                < (
-                    attrs["store"]
-                    .opening_hours.filter(weekday=attrs["pickup_time"].weekday() + 1)
-                    .first()
-                    .from_hour.hour
-                )
-            ):
-                raise serializers.ValidationError(
-                    _("Pickup time must be between store's opening hours")
-                )
+                    .exists()
+                ) and (
+                    attrs["pickup_time"].hour
+                    > (
+                        attrs["store"]
+                        .opening_hours.filter(
+                            weekday=attrs["pickup_time"].weekday() + 1
+                        )
+                        .first()
+                        .to_hour.hour
+                    )
+                    or attrs["pickup_time"].hour
+                    < (
+                        attrs["store"]
+                        .opening_hours.filter(
+                            weekday=attrs["pickup_time"].weekday() + 1
+                        )
+                        .first()
+                        .from_hour.hour
+                    )
+                ):
+                    raise serializers.ValidationError(
+                        _("Pickup time must be between store's opening hours")
+                    )
         payment_method = attrs.get("payment_method")
         if payment_method:
             if payment_method.payment_provider == store_settings.WALLET:
@@ -291,6 +308,22 @@ class OrderSerializer(serializers.ModelSerializer):
 
         return super().validate(attrs)
 
+    def perform_payment(self, amount, payment_method, order_store, orders):
+        try:
+            payment = Payment.objects.create(
+                user=self.context["request"].user,
+                amount=amount,
+                method=payment_method,
+                currency=order_store.currency,
+                orders=orders,
+            )
+        except ValidationError as err:
+            raise serializers.ValidationError(_(str(err)))
+        return {
+            "orders": orders,
+            "payment_url": payment.payment_url,
+        }
+
     def create(self, validated_data: typing.Dict):
         user = self.context["request"].user
         orders = []
@@ -310,7 +343,7 @@ class OrderSerializer(serializers.ModelSerializer):
         else:
             cart = user.cart
             amount = cart.total_price
-            stores = Store.objects.filter(store_items__cart=cart)
+            stores = Store.objects.filter(store_items__cart=cart).distinct()
             orders = []
             validated_data.pop("store")
             for store in stores:
@@ -326,17 +359,13 @@ class OrderSerializer(serializers.ModelSerializer):
                     )
                     order_item.attribute_choices.set(item.attribute_choices.all())
                 orders.append(order)
-        payment = Payment.objects.create(
-            user=user,
+        payment_method = validated_data.get("payment_method")
+        return self.perform_payment(
             amount=amount,
-            method=validated_data.get("payment_method"),
-            currency=order_store.currency,
+            payment_method=payment_method,
+            order_store=order_store,
             orders=orders,
         )
-        return {
-            "orders": orders,
-            "payment_url": payment.payment_url,
-        }
 
 
 class CreateOrderResponseSerializer(serializers.Serializer):
@@ -373,6 +402,8 @@ class ProductAttributeSerializer(serializers.ModelSerializer):
             "is_mandatory",
             "attribute_choices",
             "type",
+            "min",
+            "max",
         )
 
 
@@ -403,6 +434,8 @@ class ProductVariantSerializer(serializers.ModelSerializer):
 
 class CartItemSerializer(InventoryValidationMixin, serializers.ModelSerializer):
     image = serializers.SerializerMethodField()
+    inventory_quantity = serializers.SerializerMethodField()
+    is_uncountable = serializers.SerializerMethodField()
 
     class Meta:
         model = CartItem
@@ -418,6 +451,8 @@ class CartItemSerializer(InventoryValidationMixin, serializers.ModelSerializer):
             "extra_infos",
             "attribute_choices_total_price",
             "image",
+            "inventory_quantity",
+            "is_uncountable",
         )
         extra_kwargs = {
             "store": {
@@ -425,6 +460,16 @@ class CartItemSerializer(InventoryValidationMixin, serializers.ModelSerializer):
             },
         }
 
+    def get_inventory_quantity(self, obj):
+        if obj.inventory:
+            return obj.inventory.quantity
+        return None
+
+    def get_is_uncountable(self, obj):
+        if obj.inventory:
+            return obj.inventory.is_uncountable
+        return None
+
     def validate(self, attrs: typing.Dict) -> typing.Dict:
         return super(CartItemSerializer, self).validate(attrs)
 
@@ -470,6 +515,13 @@ class CartSerializer(serializers.ModelSerializer):
             "total_price_with_tax",
         )
 
+    def validate(self, attrs):
+        attrs = super().validate(attrs)
+        stores = set([item["store"] for item in attrs["items"]])
+        if not store_settings.DIFFERENT_STORE_ORDERING and len(stores) > 1:
+            raise ValidationError(_("You cannot order from different stores"))
+        return attrs
+
     def update(self, instance, validated_data):
         instance.items.all().delete()
         # update or create instance items
@@ -484,9 +536,16 @@ class CartSerializer(serializers.ModelSerializer):
             cart_item.save()
         return instance
 
+    def to_representation(self, instance):
+        data = super().to_representation(instance)
+        stores = Store.objects.filter(store_items__cart=instance)
+        data["store"] = StoreSerializer(stores, many=True, context=self.context).data
+        return data
+
 
 class ProductMediaSerializer(serializers.ModelSerializer):
-    image_thumbnail = serializers.ImageField(read_only=True)
+    image_thumbnail_medium = serializers.ImageField(read_only=True)
+    image_thumbnail_small = serializers.ImageField(read_only=True)
 
     class Meta:
         model = ProductMedia
@@ -494,16 +553,36 @@ class ProductMediaSerializer(serializers.ModelSerializer):
             "id",
             "is_primary",
             "image",
-            "image_thumbnail",
+            "image_thumbnail_small",
+            "image_thumbnail_medium",
             "order_value",
         )
 
 
-class ProductSerializer(serializers.ModelSerializer):
+class FavoriteMixin:
+    def to_representation(self, instance):
+        if self.context.get("request"):
+            self.favorites = self._get_favorite_object(instance)
+        else:
+            self.favorites = []
+        return super().to_representation(instance)
+
+    def _get_favorite_object(self, instance):
+        user = self.context["request"].user
+        qs = Favorite.objects.favorites_for_object(instance, user.id).values_list(
+            "id", flat=True
+        )
+        return qs
+
+    def get_favorites(self, obj):
+        return self.favorites
+
+
+class ProductSerializer(FavoriteMixin, serializers.ModelSerializer):
     product_variants = ProductVariantSerializer(many=True)
     product_images = ProductMediaSerializer(many=True, source="images")
-    is_favorite = serializers.SerializerMethodField()
     default_variant = ProductVariantSerializer(read_only=True, many=False)
+    favorites = serializers.SerializerMethodField()
 
     class Meta:
         model = Product
@@ -512,32 +591,19 @@ class ProductSerializer(serializers.ModelSerializer):
             "name",
             "slug",
             "description",
-            "is_favorite",
             "product_images",
             "product_variants",
             "default_variant",
+            "favorites",
         )
 
-    def get_is_favorite(self, obj):
-        if user := self.context["request"].user:
-            # The context manager slightly shortens the code and significantly clarifies the author's intention to ignore the specific errors.
-            # The standard library feature was introduced following a [discussion](https://bugs.python.org/issue15806), where the consensus was that
-            # A key benefit here is in the priming effect for readers...
-            # The with statement form makes it clear before you start reading the code that certain exceptions won't propagate.
-            # https://docs.python.org/3/library/contextlib.html
-            with contextlib.suppress(Favorite.DoesNotExist):
-                Favorite.objects.favorite_for_user(obj, user)
-                return True
-        return False
-
     def to_representation(self, instance: Product):
         data = super().to_representation(instance=instance)
+        data["is_favorite"] = len(self.favorites) > 0
         return data
 
 
 class ProductListSerializer(ProductSerializer):
-    product_variants = ProductVariantSerializer(many=True)
-
     class Meta:
         model = Product
         fields = (
@@ -546,15 +612,14 @@ class ProductListSerializer(ProductSerializer):
             "slug",
             "description",
             "product_images",
-            "product_variants",
             "type",
-            "default_variant",
         )
 
 
 class SubCategorySerializer(serializers.ModelSerializer):
     products = ProductListSerializer(many=True)
-    image_thumbnail = serializers.ImageField(read_only=True)
+    image_thumbnail_medium = serializers.ImageField(read_only=True)
+    image_thumbnail_small = serializers.ImageField(read_only=True)
 
     class Meta:
         model = Category
@@ -565,9 +630,15 @@ class SubCategorySerializer(serializers.ModelSerializer):
             "is_active",
             "products",
             "image",
-            "image_thumbnail",
+            "image_thumbnail_medium",
+            "image_thumbnail_small",
+            "parent",
         )
 
+    def to_representation(self, instance):
+        data = super().to_representation(instance)
+        return data
+
 
 class CategorySerializer(serializers.ModelSerializer):
     products = ProductListSerializer(many=True)
@@ -583,6 +654,9 @@ class CategorySerializer(serializers.ModelSerializer):
             "is_active",
             "subcategories",
             "parent",
+            "image",
+            "image_thumbnail_medium",
+            "image_thumbnail_small",
         )
 
 
@@ -651,6 +725,7 @@ class InventorySerializer(serializers.ModelSerializer):
             "store",
             "quantity",
             "price",
+            "plu",
             "preparation_time",
             "discount_percent",
             "discounted_price",
@@ -668,18 +743,20 @@ class PhoneContactSerializer(serializers.ModelSerializer):
         model = PhoneContact
         fields = (
             "id",
-            "phone_number",
+            "national_number",
+            "country_code",
             "is_default",
             "is_active",
         )
 
 
-class StoreSerializer(serializers.ModelSerializer):
+class StoreSerializer(FavoriteMixin, serializers.ModelSerializer):
 
-    opening_hours = OpeningHourSerializer(many=True, read_only=True)
+    opening_hours = serializers.SerializerMethodField()
     phone_contacts = serializers.SerializerMethodField()
     in_range_delivery = serializers.SerializerMethodField()
     is_closed = serializers.SerializerMethodField()
+    favorites = serializers.SerializerMethodField()
     is_favorite = serializers.SerializerMethodField()
     address_line = serializers.SerializerMethodField()
     shipping_methods = ShippingMethodSerializer(many=True, read_only=True)
@@ -704,12 +781,17 @@ class StoreSerializer(serializers.ModelSerializer):
             "opening_hours",
             "in_range_delivery",
             "is_closed",
+            "favorites",
             "is_favorite",
             "created_at",
             "updated_at",
             "phone_contacts",
             "current_day_opening_hours",
+            "image",
         )
+        extra_kwargs = {
+            "image": {"read_only": True, "required": False},
+        }
 
     def get_is_closed(self, obj):
         current_time = now()
@@ -726,22 +808,43 @@ class StoreSerializer(serializers.ModelSerializer):
 
     def get_in_range_delivery(self, obj):
         user_location = self.context["request"].query_params.get("point")
+        in_range_method = obj.shipping_methods.filter(
+            type=ShippingMethod.ShippingType.DELIVERY
+        ).exists()
+        if not in_range_method:
+            return True
         if user_location and obj.poly:
             long, lat = user_location.split(",")
             return obj.poly.contains(Point(float(long), float(lat)))
-        return False
+
+    def get_opening_hours(self, obj):
+        opening_hours = sorted(
+            list(obj.opening_hours.all()), key=lambda op_hour: op_hour.weekday
+        )
+        formatted_hours = []
+        if len(opening_hours) > 0:
+            day_1 = calendar.day_name[opening_hours[0].weekday - 1]
+            day_2 = None
+            for i in range(len(opening_hours)):
+                if not opening_hours[i] == opening_hours[-1]:
+                    if (
+                        opening_hours[i].from_hour == opening_hours[i + 1].from_hour
+                        and opening_hours[i].to_hour == opening_hours[i + 1].to_hour
+                    ):
+                        day_2 = calendar.day_name[opening_hours[i + 1].weekday - 1]
+                        continue
+                formatted_days = day_1
+                if day_2:
+                    formatted_days = f"{formatted_days} - {day_2}"
+                formatted_hours.append(
+                    f"{formatted_days} {opening_hours[i].from_hour.strftime('%I:%M%p')} - {opening_hours[i].to_hour.strftime('%I:%M%p')}"
+                )
+                if not opening_hours[i] == opening_hours[-1]:
+                    day_1 = calendar.day_name[opening_hours[i + 1].weekday - 1]
+        return formatted_hours
 
     def get_is_favorite(self, obj):
-        if user := self.context["request"].user:
-            # The context manager slightly shortens the code and significantly clarifies the author's intention to ignore the specific errors.
-            # The standard library feature was introduced following a [discussion](https://bugs.python.org/issue15806), where the consensus was that
-            # A key benefit here is in the priming effect for readers...
-            # The with statement form makes it clear before you start reading the code that certain exceptions won't propagate.
-            # https://docs.python.org/3/library/contextlib.html
-            with contextlib.suppress(Favorite.DoesNotExist):
-                Favorite.objects.favorite_for_user(obj, user)
-                return True
-        return False
+        return len(self.favorites) > 0
 
     def get_address_line(self, obj):
         return obj.address.address_line
@@ -750,12 +853,14 @@ class StoreSerializer(serializers.ModelSerializer):
         # get the distance between the user location and store location
         user_location = self.context["request"].query_params.get("point")
         if user_location and obj.location:
-
+            unit = "km"
             lat, long = user_location.split(",")
             store_lat, store_long = obj.location.x, obj.location.y
-            return round(
-                distance((float(lat), float(long)), (store_lat, store_long)), 1
-            )
+            ds = round(distance((float(lat), float(long)), (store_lat, store_long)), 1)
+            if ds < 1:
+                ds = int(ds * 1000)
+                unit = "m"
+            return f"{ds}{unit}"
 
     def get_phone_contacts(self, obj):
         phone_contacts = obj.phone_contacts.filter(is_active=True).order_by(
@@ -764,25 +869,20 @@ class StoreSerializer(serializers.ModelSerializer):
         return PhoneContactSerializer(phone_contacts, many=True).data
 
     def get_current_day_opening_hours(self, obj):
-        for op_hour in obj.opening_hours.all().only("id"):
-            if op_hour.weekday == date.today().weekday():
-                return OpeningHourSerializer(op_hour).data
-            else:
-                op_hour = OpeningHourSerializer(op_hour).data
-                op_hour["from_hour"] = settings.DEFAULT_OPENING_HOURS[0]["from_hour"]
-                op_hour["to_hour"] = settings.DEFAULT_OPENING_HOURS[0]["to_hour"]
-                return op_hour
+        current_opening_hours = obj.opening_hours.filter(
+            weekday=now().weekday() + 1
+        ).first()
+        op_hour = {}
+        if current_opening_hours:
+            op_hour["from_hour"] = current_opening_hours.from_hour.strftime("%I:%M%p")
+            op_hour["to_hour"] = current_opening_hours.to_hour.strftime("%I:%M%p")
+        else:
+            op_hour["from_hour"] = settings.DEFAULT_OPENING_HOURS[0]["from_hour"]
+            op_hour["to_hour"] = settings.DEFAULT_OPENING_HOURS[0]["to_hour"]
+        return op_hour
 
     def to_representation(self, instance):
-        data = super().to_representation(instance)
-        view = self.context.get("view")
-        if view:
-            if view.action != "retrieve":
-                return data
-        data["inventories"] = InventorySerializer(
-            instance.inventories.all(), many=True
-        ).data
-        return data
+        return super().to_representation(instance)
 
 
 class PaymentMethodSerializer(serializers.ModelSerializer):
@@ -822,17 +922,27 @@ class StoreListSerializer(serializers.ModelSerializer):
 
 
 class GenericSerializer(serializers.Serializer):
+    """
+    A generic serializer that automatically selects the appropriate serializer
+    for a given instance.
+
+    The `to_representation` method uses the `get_serializer_for_instance` method
+    to determine which serializer to use for a given instance. If the serializer
+    class is found, it returns the representation of the instance using that
+    serializer. Otherwise, it raises a `NameError`.
+    """
+
     def to_representation(self, value):
         context = self.context
         serializer_class = self.get_serializer_for_instance(value)
         return serializer_class(context=context).to_representation(value)
 
     def get_serializer_for_instance(self, instance):
-        serializer_name = type(instance).__name__ + "Serializer"
-        try:
-            return eval(serializer_name)
-        except NameError:
-            raise NameError(_(f"name '{serializer_name}' is not defined"))
+        serializer_class = instance.__class__.__name__ + "Serializer"
+        if hasattr(sys.modules[__name__], serializer_class):
+            return getattr(sys.modules[__name__], serializer_class)
+        else:
+            raise NameError(_(f"name '{serializer_class}' is not defined"))
 
 
 class FavoriteExtraSerializer(serializers.ModelSerializer):
@@ -870,15 +980,41 @@ class FavoriteSerializer(serializers.ModelSerializer):
             "object_type",
         )
 
+    def _lookup_validation(self, data):
+        content_type = ContentType.objects.get_for_model(type(data["content_object"]))
+        extras_lookup_content_type = {}
+        for extra in data["extras"]:
+            content_type_extra = extra.model
+            extras_lookup_content_type[content_type_extra] = extra
+        queryset = Favorite.objects.filter(
+            content_type=content_type,
+            object_id=data["content_object"].id,
+            user=self.context["request"].user,
+        )
+        for key in extras_lookup_content_type.keys():
+            queryset = queryset.filter(
+                extras__content_type=ContentType.objects.get_for_model(key),
+                extras__object_id__in=[
+                    extra.id for extra in extras_lookup_content_type[key]
+                ],
+            )
+        if queryset.exists():
+            raise serializers.ValidationError(
+                _(f"You cannot favorite the same item twice")
+            )
+
     def validate(self, attrs):
         validated_data = super().validate(attrs)
         object_type = validated_data["object_type"]
-        if object_type not in store_settings.FAVORITE_TYPES:
+        if object_type not in store_settings.FAVORITE_TYPES and hasattr(
+            sys.modules[__name__], object_type
+        ):
             raise serializers.ValidationError(
                 _(f"Cannot resolve keyword '{object_type}' into an object type")
             )
+        object_type_model = getattr(sys.modules[__name__], object_type)
         object_instance = get_object_or_404(
-            eval(object_type), pk=validated_data["object_id"]
+            object_type_model, pk=validated_data["object_id"]
         )
         extras_data = {}
         extras = []
@@ -894,16 +1030,17 @@ class FavoriteSerializer(serializers.ModelSerializer):
             extras_data[extra_object_type].append(extra["object_id"])
         try:
             for key in extras_data.keys():
-                model_class = eval(key)
-                extras.append(model_class.objects.filter(id__in=extras_data[key]))
+                model_class = getattr(sys.modules[__name__], key, None)
+                if model_class:
+                    extras.append(model_class.objects.filter(id__in=extras_data[key]))
         except ObjectDoesNotExist:
             raise serializers.ValidationError(_("No matches the given query."))
-        except NameError as e:
-            raise NameError(e)
+
         validated_data = {
             "content_object": object_instance,
             "extras": extras,
         }
+        self._lookup_validation(validated_data)
         return validated_data
 
     def create(self, validated_data):
