@@ -1,8 +1,11 @@
 import logging
 import typing
 
+from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
 from django.db.models import Prefetch
+from django.http import Http404
+from django.shortcuts import get_object_or_404
 from django.utils.decorators import method_decorator
 from django.utils.translation import ugettext_lazy as _
 from django_auto_prefetching import AutoPrefetchViewSetMixin, prefetch
@@ -20,6 +23,7 @@ from rest_framework_nested.viewsets import NestedViewSetMixin
 from config import settings as store_settings
 from ob_dj_store.apis.stores.filters import (
     CategoryFilter,
+    FavoriteFilter,
     InventoryFilter,
     OrderFilter,
     ProductFilter,
@@ -78,9 +82,10 @@ class StoreView(
     queryset = Store.objects.active()
     distance_ordering_filter_field = "location"
     filter_backends = [DistanceToPointOrderingFilter, DjangoFilterBackend]
+    lookup_value_regex = "[0-9]+"
 
     def get_permissions(self):
-        if self.action in ["favorites", "favorite", "recently_ordered_from"]:
+        if self.action in ["favorites", "favorite", "recently_ordered_from", "count"]:
             return [
                 permissions.IsAuthenticated(),
             ]
@@ -206,6 +211,81 @@ class StoreView(
         serializer = StoreSerializer(instance=instance, context={"request": request})
         return Response(serializer.data)
 
+    @swagger_auto_schema(
+        operation_summary="Retrieve count of store's products",
+        operation_description="""
+            Retrieve count of store's products
+        """,
+        tags=[
+            "Store",
+        ],
+    )
+    @action(
+        detail=True,
+        methods=["GET"],
+        url_path="menu-count",
+    )
+    def menu_count(self, request, *args, **kwargs):
+        store = self.get_object()
+        products_ids = Product.objects.filter(
+            product_variants__inventories__store=store
+        ).values_list("id", flat=True)
+        content_type = ContentType.objects.get_for_model(Product)
+        favorites_count = Favorite.objects.filter(
+            content_type=content_type,
+            object_id__in=products_ids,
+        ).count()
+        menu = Product.objects.filter(
+            product_variants__inventories__store=store,
+            category__isnull=False,
+            is_active=True,
+        )
+        data = {
+            "favorites_count": favorites_count,
+            "menu_count": menu.count(),
+            "featured_count": menu.filter(is_featured=True).count(),
+        }
+        return Response(data, status=status.HTTP_200_OK)
+
+    @swagger_auto_schema(
+        operation_summary="Retrieve count of stores",
+        operation_description="""
+            Retrieve count of stores (Nearby,previous,favorites)
+        """,
+        tags=[
+            "Store",
+        ],
+    )
+    @action(
+        detail=False,
+        methods=["GET"],
+        url_path="count",
+    )
+    def count(self, request, *args, **kwargs):
+        previous_count = (
+            super()
+            .get_queryset()
+            .filter(
+                orders__customer=self.request.user,
+                orders__status__in=[
+                    "PAID",
+                    "DELIVERED",
+                ],
+            )
+            .count()
+        )
+        nearby_count = super().get_queryset().count()
+        favorite_store_ids = Favorite.objects.favorites_for_model(
+            Store, self.request.user
+        ).values_list("object_id", flat=True)
+        favorites_count = self.queryset.filter(pk__in=favorite_store_ids).count()
+        data = {
+            "previous_count": previous_count,
+            "nearby_count": nearby_count,
+            "favorites_count": favorites_count,
+        }
+        return Response(data, status=status.HTTP_200_OK)
+
 
 class CartView(
     mixins.RetrieveModelMixin, mixins.UpdateModelMixin, viewsets.GenericViewSet
@@ -215,6 +295,7 @@ class CartView(
         permissions.IsAuthenticated,
     ]
     queryset = Cart.objects.all()
+    lookup_value_regex = "[0-9]+"
 
     def get_object(self):
         return self.request.user.cart
@@ -255,6 +336,7 @@ class CartItemView(
         permissions.IsAuthenticated,
     ]
     queryset = CartItem.objects.all()
+    lookup_value_regex = "[0-9]+"
 
     @swagger_auto_schema(
         operation_summary="Retrieve Cart Item",
@@ -395,6 +477,40 @@ class OrderView(
             )
         return Response(serializer.data, status=status.HTTP_200_OK)
 
+    @swagger_auto_schema(
+        operation_summary="re-order a Customer Order",
+        operation_description="""
+            re-order a customer order
+        """,
+        tags=[
+            "Order",
+        ],
+    )
+    @action(
+        methods=["POST"],
+        detail=True,
+        url_path=r"reorder",
+        permission_classes=[
+            permissions.IsAuthenticated,
+        ],
+    )
+    def reorder(self, request: Request, *args: typing.Any, **kwargs: typing.Any):
+        order = self.get_object()
+        request.user.cart.fill(order)
+        context = self.get_serializer_context()
+        serializer = OrderSerializer(data=request.data, context=context)
+        orders = []
+        serializer.is_valid(raise_exception=True)
+        orders, payment_url = serializer.save().values()
+        orders_data = OrderSerializer(orders, many=True).data
+        return Response(
+            status=status.HTTP_201_CREATED,
+            data={
+                "payment_url": payment_url,
+                "orders": orders_data,
+            },
+        )
+
 
 class VariantView(mixins.ListModelMixin, viewsets.GenericViewSet):
     serializer_class = ProductVariantSerializer
@@ -450,10 +566,12 @@ class ProductView(
             queryset = self.queryset.filter(pk__in=favorite_product_ids)
         return queryset
 
-    def get_serializer_class(self):
-        # # TODO: Replace If logic with dict lookup
-        # Listing Serializer
-        return ProductListSerializer if self.action == "list" else ProductSerializer
+    def get_object(self):
+
+        instance = self.get_queryset().filter(pk=self.kwargs["pk"]).distinct().first()
+        if not instance:
+            raise Http404("No Product matches the given query.")
+        return instance
 
     @swagger_auto_schema(
         operation_summary="Retrieve A Product",
@@ -552,6 +670,37 @@ class CategoryViewSet(
     permission_classes = (permissions.AllowAny,)
     queryset = Category.objects.active()
     filterset_class = CategoryFilter
+    lookup_value_regex = "[0-9]+"
+
+    def get_object(self):
+        store_id = self.request.query_params.get("store", None)
+        instance_pk = self.kwargs["pk"]
+        if not store_id:
+            return get_object_or_404(Category, pk=instance_pk, is_active=True)
+        try:
+            product_queryset = Product.objects.filter(
+                product_variants__inventories__store=store_id,
+                is_active=True,
+            )
+            instance = self.queryset.prefetch_related(
+                Prefetch(
+                    "subcategories",
+                    queryset=Category.objects.filter(
+                        products__product_variants__inventories__store=store_id
+                    ).distinct(),
+                ),
+                Prefetch(
+                    "subcategories__products",
+                    queryset=product_queryset.distinct(),
+                ),
+                Prefetch(
+                    "products",
+                    queryset=product_queryset.distinct(),
+                ),
+            ).get(pk=instance_pk)
+        except Category.DoesNotExist:
+            raise Http404("No Category matches the given query.")
+        return instance
 
     @method_decorator(
         name="retrieve",
@@ -626,7 +775,7 @@ class TransactionsViewSet(
     def get_queryset(self):
         return Payment.objects.filter(
             user=self.request.user, status=Payment.PaymentStatus.SUCCESS.value
-        ).order_by("payment_post_at")
+        ).order_by("-payment_post_at")
 
     @swagger_auto_schema(
         operation_summary="List Users's Captured Transactions",
@@ -740,13 +889,18 @@ class FavoriteViewSet(
         permissions.IsAuthenticated,
     ]
     queryset = Favorite.objects.all()
+    filterset_class = FavoriteFilter
 
     def get_queryset(self):
         model = self.request.query_params.get("type")
+        if self.action == "destroy":
+            return self.queryset.filter(user=self.request.user)
         if not model and model not in store_settings.FAVORITE_TYPES:
-            raise ValidationError(_("You must"))
-        queryset = Favorite.objects.favorites_for_model(eval(model), self.request.user)
-        return queryset
+            raise ValidationError(_("You must provide the favorite type"))
+        self.queryset = Favorite.objects.favorites_for_model(
+            eval(model), self.request.user
+        )
+        return super().get_queryset()
 
     @swagger_auto_schema(
         operation_summary="List Favorites",
