@@ -78,6 +78,7 @@ class StoreAdmin(LeafletGeoAdmin):
                     "minimum_order_amount",
                     "payment_methods",
                     "pickup_addresses",
+                    "image",
                 )
             },
         ),
@@ -104,6 +105,12 @@ class CategoryAdmin(admin.ModelAdmin):
         "is_active",
     ]
 
+    def save_model(self, request, obj, form, change) -> None:
+        from ob_dj_store.core.stores.receivers import create_media_thumbnails
+
+        create_media_thumbnails(None, obj, None)
+        return super().save_model(request, obj, form, change)
+
 
 class ProductVariantAdmin(admin.ModelAdmin):
     inlines = [
