@@ -32,7 +32,7 @@ def initiate_payment(
     callback_url = f"{settings.WEBSITE_URI}{callback_path}"
 
     payload = {
-        "amount": "%.3f" % payment.amount,
+        "amount": "%.3f" % payment.total_payment,
         "currency": currency_code,
         "source": {"id": source},
         "customer": {
@@ -59,15 +59,15 @@ def initiate_payment(
         url=f"{settings.TAP_API_URL}{url}", method=method, data=payload, headers=headers
     )
     tap_response = response.json()
-    payment_url = tap_response.get("transaction").get("url")
+    payment_transaction = tap_response.get("transaction", None)
     charge_id = tap_response.get("id")
-    if not payment_url or not charge_id:
+    if not payment_transaction or not charge_id:
         # TODO: How does this issue occur and is this the best way to handle it?
         logger.error(
             f"Failed to create charge request no payment_url or charge_id returned."
         )
         raise TapException(payload)
-
+    payment_url = payment_transaction.get("url")
     status = tap_response.get("status")
     source = tap_response.get("source").get("id") if source else ""
 
