@@ -26,7 +26,6 @@ class Favorite(DjangoModelCleanMixin, models.Model):
     class Meta:
         verbose_name = _("favorite")
         verbose_name_plural = _("favorites")
-        unique_together = (("user", "content_type", "object_id"),)
 
     def __str__(self):
         return f"{self.user} favorites {self.content_object}"
@@ -66,8 +65,5 @@ class FavoriteExtra(DjangoModelCleanMixin, models.Model):
 
     objects = FavoriteExtraManager()
 
-    class Meta:
-        unique_together = (("favorite", "content_type", "object_id"),)
-
     def __str__(self):
         return f"{self.content_object}"
