@@ -82,6 +82,8 @@ class Order(DjangoModelCleanMixin, models.Model):
     pickup_time = models.DateTimeField(
         null=True, blank=True, help_text=_("Pickup time")
     )
+    # Order id of the pickup_car
+    car_id = models.PositiveIntegerField(null=True, blank=True)
     # Pick up can be now or a later hour during the day. If pickup_time is not set,
     extra_infos = models.JSONField(null=True, blank=True)
 
@@ -155,7 +157,7 @@ class OrderItem(DjangoModelCleanMixin, models.Model):
     product_variant = models.ForeignKey(
         "stores.ProductVariant",
         null=True,
-        on_delete=models.SET_NULL,
+        on_delete=models.PROTECT,
         related_name="order_items",
     )
     # notes for special instructions, can be empty
@@ -179,7 +181,7 @@ class OrderItem(DjangoModelCleanMixin, models.Model):
     extra_infos = models.JSONField(null=True, blank=True)
 
     def __str__(self):
-        return f"OrderItem - {self.quantity} {self.product_variant.name}"
+        return f"OrderItem - {self.quantity} {self.product_variant.product.name}{self.product_variant.name}"
 
     class Meta:
         verbose_name = _("Order Item")
