@@ -12,6 +12,7 @@ from ob_dj_store.core.stores.managers import (
     StoreManager,
 )
 from ob_dj_store.core.stores.utils import get_currency_by_country
+from ob_dj_store.utils.helpers import store_media_upload_to
 from ob_dj_store.utils.model import DjangoModelCleanMixin
 
 
@@ -134,6 +135,7 @@ class Store(DjangoModelCleanMixin, models.Model):
         default=0,
         help_text=_("This is the min price to get a free delivery"),
     )
+    image = models.ImageField(upload_to=store_media_upload_to, null=True, blank=True)
 
     created_at = models.DateTimeField(auto_now_add=True)
     updated_at = models.DateTimeField(auto_now=True)
@@ -219,3 +221,11 @@ class PhoneContact(DjangoModelCleanMixin, models.Model):
 
     def __str__(self) -> str:
         return f"Phone Contacts for store {self.store}"
+
+    @property
+    def country_code(self):
+        return self.phone_number.country_code
+
+    @property
+    def national_number(self):
+        return self.phone_number.national_number
