@@ -26,3 +26,9 @@ def category_media_upload_to(instance, filename):
     ext = filename.split(".")[-1]
     if instance:
         return f"category_media/{instance.name}_{instance.order_value}_{int(now().timestamp())}.{ext}"
+
+
+def store_media_upload_to(instance, filename):
+    ext = filename.split(".")[-1]
+    if instance:
+        return f"store_media/{instance.name}_{int(now().timestamp())}.{ext}"
